#ifndef __DECLROM_H__

#include <stdint.h>
#include <sys/queue.h>

#define kDeclRomPattern 0x5a932bc7

struct sRsrcType {
	uint16_t category;
	uint16_t cType;
	uint16_t drSW;
	uint16_t drHW;
};
typedef struct sRsrcType sRsrcType;

struct sBlock {
	uint32_t len;
	uint8_t *data;
};
typedef struct sBlock sBlock;

struct sExecBlock {
	uint32_t len;
	uint8_t revision;
	uint8_t cpuid;
	uint32_t offset;
	uint8_t *code;
};
typedef struct sExecBlock sExecBlock;

struct sDriver {
	uint32_t len;
	uint8_t *data;

	uint8_t cpuid;
};
typedef struct sDriver sDriver;

struct sResource {
	uint8_t rsrcid;
	/* required */
	sRsrcType *type;
	char *name;
	
	/* optional */
	sDriver *drvrDir[4];
	sExecBlock *loadRec;
	sExecBlock *bootRec;
	uint32_t flags;
	uint32_t devid;;
	
	uint16_t boardid;
	sExecBlock *primaryInit;
	
	char *vendorid;
	char *serialnum;
	char *revlevel;
	char *partnum;
	char *date;

	uint32_t *minBaseOS;
	uint32_t *minLen;

	uint32_t *unknowns;
	uint32_t unknowncount;

	TAILQ_ENTRY(sResource) next;
};
typedef struct sResource sResource;

struct sFormatBlock {
	uint8_t bytelanes;
	uint8_t reserved;
	uint32_t testpattern;
	uint8_t format;
	uint8_t revision;
	uint32_t crc;
	uint32_t length;
	int32_t diroffset;
};
typedef struct sFormatBlock sFormatBlock;

struct sDeclRom {
	sFormatBlock	formatblock;
	TAILQ_HEAD(rsrcq, sResource) resourcelist;
	uint8_t *rom;
	uint32_t romlen;
};
typedef struct sDeclRom sDeclRom;

void sdeclrom_free(sDeclRom *rom);
int sdeclrom_writefile(sDeclRom *rom, char *file);
sDeclRom *sdeclrom_readfile(char *infile);

#endif /* __DECLROM_H__ */
